(define-module (alterator menu)

  :use-module (srfi srfi-1)
  :use-module (srfi srfi-2)
  :use-module (srfi srfi-11)
  :use-module (srfi srfi-13)

  :use-module (alterator str)
  :use-module (alterator algo)
  :use-module (alterator plist)
  :use-module (alterator woo)
  :use-module (alterator gettext)

  :use-module (alterator ensign)

  :use-module (alterator http html)
  :use-module (alterator http template)

  :use-module (alterator configd html)
  :use-module (alterator configd frontend)
  :export (menu))

;;; helper functions

(define (tag? name)
  (lambda(x)
    (and (pair? x)
         (eq? (car x) name)
	 x)))

(define (try-unquote scm)
  (if (eq? (car scm) 'quote)
      (cdr scm)
      scm))

(define (help-referrer url)
  (if (string-prefix? "/help" url)
      (substring url (string-length "/help"))
      url))

;;; help link

(define (help-tag url url-args _)
  (let ((url (if (string=? url "/") "/index" url)))
    (tag: "div"
          (@ 'class "help")
          (if (string-prefix? "/help" url)
              `(local:a ,(@ 'href (help-referrer url)) ,(_ "Return"))
              `(local:a ,(@ 'href (string-append "/help" url)) ,(_ "Help"))))))

;;; menu tree

(define (menu-item url)
  (let ((url (string-sure-slash url)))
    (lambda(x)
      (let ((item-url (string-sure-slash (car x))))
        (html: "div"
               (@ 'class (if (string-prefix? item-url url) "menu-item-selected" "menu-item-normal"))
               (html: "local:a" (cdr x) (@ 'href item-url)))))))

(define (menu-group url language)
  (lambda(x res)
    (let ((items (woo-list/name+label (string-append "/menu/" (car x)) 'language language 'ui "html")))
      (if (pair? items)
          (append1 res (html: "div"
                              (@ 'class "menu-group")
                              (html: "div" (@ 'class "menu-group-name") (cdr x))
                              (map (menu-item url) items)))
          res))))

(define (menu-tag url language)
  (tag: "div"
        (@ 'class "menu")
        (fold (menu-group url language)
              '()
              (woo-list/name+label "/menu" 'language language 'ui "html"))))

;;; head title

(define (title/info url url-args)
  (woo-get-option (woo-read-first (string-append "/index/" (base-path url))
                                  'language (cond-assoc "language" url-args '("en_US")))
                  'title))
        
(define (title/tag tag)
  (and tag (apply string-append (filter string? tag))))

(define (std-title std name)
  (if (string-null? name)
      std
      (string-append name " - " std)))

(define (head-tag url url-args head _)
  (let-values ( ((title rest) (partition (tag? 'title) (cdr head))) )
    (tag: "head"
          (@ 'template-operation 'append-content)
          (html: "title"
                 (or (title/tag (cond-car title))
                     (std-title (_ "ALT Linux Console") (title/info url url-args))))
          rest)))

;;; screen body
(define (main-tag body)
  (tag: "div"
        (@ 'class "main")
          (cdr body)))

;;; main
;;generate main menu
(define (apply-menu  scm design-url url url-args)
  (or (and-let* (((not (eq? (car scm) 'quote)))
                 (html (any (tag? 'html) scm))
                 (head (any (tag? 'head) html))
                 (body (any (tag? 'body) html))
                 (design-url (design-path design-url))
                 (language (cond-assoc "language" url-args '("en_US")))
                 (_ (make-translator "alterator-fbi" language)))

                (template design-url
                          (head-tag url url-args head _)
                          (help-tag url url-args _)
                          (menu-tag (help-referrer url) language)
                          (main-tag body)))
      (try-unquote scm)))

;;method, url, args
(define (menu-main design-url cmds next)
  (let*-values (( (method url url-args . rest) (apply values (car cmds))))
    (catch #t
      (thunk (let  ((result (next cmds)))
               (if (or (cond-assoc "async" url-args)
                       (string-ci=? method "woo"))
                   result
                   (apply-menu result design-url url url-args))))
      (html:exception url-args))))

(define (menu design-url)
  (lambda (cmds next)
    (catch #t
      (thunk
       (with-fluids
        ((woo-gate ensign-gate))
        (menu-main design-url cmds next)))
      (html:exception '() ))))
